#include <stdio.h>
#include "palabras.c"
#include "rw_mp.h"
#include "transf_inst.h"
#include "op_logic.h"
#include "cmp.h"
#include "secl_flags.h"
#include "salto_inst.h"
#include "aritm_inst.h"
#include "6502core.h"


inline union tipo_palabra8* generar_excepciones(struct cpu6502* cpu);

int inline ejecutar(int *halt,struct cpu6502* cpu, unsigned long* ciclo)

{

  //for (i=0; i<n; i++)
  // {
  leer_mp(&cpu->IR,cpu->memory_map,cpu->PC);  //IR <- [PC] FETCH
  ciclo[0] +=  4; //Promedio de ciclos que dura una instruccion.


  switch (cpu->IR.uns)
  {
  case 0x00: //BRK A
    cpu->brk_irq = 2;
    cpu->PC.uns += 2;  //FCEU lo incrementa en 2.
    break;

    //ASLs
  case 0x0A: //ASL A r1 ok
    asl_a(&cpu->PC,&cpu->A,&cpu->P);
    break;
  case 0x06: //ASL aa r1 ok
    asl_z(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0x16: //ASL a&cpu->A, X
    asl_xz(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x0E: //ASL aaaa
    asl(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0x1E: //ASL aaa&cpu->A, X
    asl_indx(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;

    //LSRs
  case 0x4A: //LSR A
    lsr_a(&cpu->PC,&cpu->A,&cpu->P);
    break;
  case 0x46: //LSR aa
    lsr_z(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0x56: //LSR a&cpu->A, X
    lsr_xz(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x4E: //LSR aaaa
    lsr(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0x5E: //LSR aaa&cpu->A, X
    lsr_indx(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;

    //ROLs
  case 0x2A: //ROL A
    rol_a(&cpu->PC,&cpu->A,&cpu->P);
    break;
  case 0x26: //ROL aa
    rol_z(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0x36: //ROL a&cpu->A, X
    rol_xz(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x2E: //ROL aaaa
    rol(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0x3E: //ROL aaa&cpu->A, X
    rol_indx(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;

    //RORs
  case 0x6A: //ROR A
    ror_a(&cpu->PC,&cpu->A,&cpu->P);
    break;
  case 0x66: //ROR aa
    ror_z(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0x76: //ROR a&cpu->A, X
    ror_xz(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x6E: //ROR aaaa
    ror(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0x7E: //ROR aaa&cpu->A, X
    ror_indx(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;

    //cmp
  case 0xC9: //cmp #aa
    cmp_inm(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0xC5: //cmp aa
    cmp_z(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0xD5: //cmp a&cpu->A,X
    cmp_indxz(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xCD: //cmp aaaa
    cmp(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0xDD: //cmp aaa&cpu->A,X
    cmp_ind(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xD9: //cmp aaa&cpu->A,Y
    cmp_ind(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0xC1: //cmp (a&cpu->A,X)
    cmp_pre_ind(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xD1: //cmp (aa),Y
    //cmp_post_ind(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);//COMOR???
    cmp_post_ind(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;

    //CPX

  case 0xE0: //CPX #aa
    cmp_inm(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xE4: //CPX aa
    cmp_z(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xEC: //CPX aaaa
    cmp(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;

    //CPY

  case 0xC0: //CPY #aa
    cmp_inm(&cpu->PC,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0xC4: //CPY aa
    cmp_z(&cpu->PC,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0xCC: //CPY aaaa
    cmp(&cpu->PC,&cpu->Y,&cpu->P,cpu->memory_map);
    break;


    //PUSHes

  case 0x48: //PHA
    push(&cpu->PC,&cpu->A,&cpu->SP,cpu->memory_map);
    break;
  case 0x08: //PHP
    push_sr(&cpu->PC,&cpu->P,&cpu->SP,cpu->memory_map);
    break;

    //POPes
  case 0x68:  //POP_A
    pop_a(&cpu->PC,&cpu->A,&cpu->P,&cpu->SP,cpu->memory_map);
    break;
  case 0x28:  //POP_SR
    pop_sr(&cpu->PC,&cpu->P,&cpu->SP,cpu->memory_map);
    break;

    //ANDs
  case 0x29: //and #aa
    and_inm(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x25: //AND $aa
    and_zero(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x35: //AND $a&cpu->A,X
    and_indxz(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x2D: //AND $aaaa
    fand(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x3D: //AND $aaa&cpu->A,X
    and_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x39: //AND $aaa&cpu->A,Y
    and_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0x21: //AND ($a&cpu->A,X)
    and_pre_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x31: //AND ($aa), Y
    and_post_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;

    //FIN ANDs

    //ORs
  case 0x09: //OR #aa
    or_inm(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x05: //OR $aa
    or_zero(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x15: //OR $a&cpu->A,X
    or_indxz(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x0D: //OR $aaaa
    f_or(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x1D: //OR $aaa&cpu->A,X
    or_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x19: //OR $aaa&cpu->A,Y
    or_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0x01: //OR ($a&cpu->A,X)
    or_pre_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x11: //OR ($aa), Y
    or_post_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;


    //EORs

  case 0x49: //EOR #aa
    eor_inm(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x45: //EOR $aa
    eor_zero(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    //printf("puede q petase por esto\n");
    break;
  case 0x55: //EOR $a&cpu->A,X
    eor_indxz(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x4D: //EOR $aaaa
    eor(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x5D: //EOR $aaa&cpu->A,X
    eor_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x59: //EOR $aaa&cpu->A,Y
    eor_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0x41: //EOR ($a&cpu->A,X)
    eor_pre_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x51: //EOR ($aa), Y
    eor_post_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;

    //BIT
  case 0x24: //BIT aa
    bit_z(&cpu->PC, &cpu->A, &cpu->P, cpu->memory_map);
    break;
  case 0x2C:  //BIT aaaa
    bit(&cpu->PC, &cpu->A, &cpu->P, cpu->memory_map);
    break;


    //Saltos

  case 0x20: //jsr
    jsr(&cpu->PC, &cpu->SP, cpu->memory_map);
    break;
  case 0x40:
    rti(&cpu->PC, &cpu->SP, &cpu->P, cpu->memory_map);
    //printf("Vuelvo de RTI\n");
    break;
  case 0x4C: //jmp $aaaa
    jmp(&cpu->PC, cpu->memory_map);
    break;
  case 0x6C: //(jmp $aaaa)
    jmp_ind(&cpu->PC, cpu->memory_map);
    break;
  case 0x60:
    rts(&cpu->PC, &cpu->SP, cpu->memory_map);
    break;

  case 0x78: //SEI
    set(2,&cpu->PC,&cpu->P);
    break;

  case 0xF8: //SED
    set(3,&cpu->PC,&cpu->P);
    break;

  case 0x38: //SEC
    set(0,&cpu->PC,&cpu->P);
    break;

  case 0x18: //CLC
    clr(0,&cpu->PC,&cpu->P);
    break;




  case 0x90: //BCC
    bcx(&cpu->PC,&cpu->P,0,0,cpu->memory_map);
    break;
  case 0xB0: //BCS
    bcx(&cpu->PC,&cpu->P,0,1,cpu->memory_map);
    break;
  case 0xF0: //BEQ
    bcx(&cpu->PC,&cpu->P,1,1,cpu->memory_map);
    break;
  case 0x30: //BMI
    bcx(&cpu->PC,&cpu->P,7,1,cpu->memory_map);
    break;
  case 0xD0: //BNE
    bcx(&cpu->PC,&cpu->P,1,0,cpu->memory_map);
    break;
  case 0x10: //BPL
    bcx(&cpu->PC,&cpu->P,7,0,cpu->memory_map);
    break;
  case 0x50: //BVC
    bcx(&cpu->PC,&cpu->P,6,0,cpu->memory_map);
    //system("pause");
    break;
  case 0x70: //BVS
    bcx(&cpu->PC,&cpu->P,6,1,cpu->memory_map);
    //system("pause");
    break;

  case 0xD8: //CLD
    clr(3,&cpu->PC,&cpu->P);
    break;

  case 0x58: //CLI
    clr(2,&cpu->PC,&cpu->P);
    break;

  case 0xB8: //CLV
    clr(6,&cpu->PC,&cpu->P);
    break;

  case 0x86: //STX $aa
    stx_zero(&cpu->PC,&cpu->X,cpu->memory_map);
    break;
  case 0x8E: //STX $aaaa
    stx(&cpu->PC,&cpu->X,cpu->memory_map);
    break;

  case 0x84: //STY $aa
    stx_zero(&cpu->PC,&cpu->Y,cpu->memory_map);
    break;
  case 0x8C: //STY $aaaa
    stx(&cpu->PC,&cpu->Y,cpu->memory_map);
    break;

  case 0x81: //STA ($a&cpu->A,X)
    stx_pre_indx(&cpu->PC,&cpu->A,&cpu->X,cpu->memory_map);
    break;

  case 0x85: //STA $aa
    stx_zero(&cpu->PC,&cpu->A,cpu->memory_map);
    break;
  case 0x8D: //STA $aaaa
    //printf("llego a este store\n");
    stx(&cpu->PC,&cpu->A,cpu->memory_map);
    //printf("y salgo de este store\n");
    break;

  case 0x91: //STA ($aa),Y
    stx_post_indx(&cpu->PC,&cpu->A,&cpu->Y,cpu->memory_map);
    break;

  case 0x96: //STX $a&cpu->A,Y
    stx_indxZ(&cpu->PC,&cpu->X,&cpu->Y,cpu->memory_map);
    break;
  case 0x95: //STA $a&cpu->A,X
    stx_indxZ(&cpu->PC,&cpu->A,&cpu->X,cpu->memory_map);
    break;
  case 0x94: //STY $a&cpu->A,X
    stx_indxZ(&cpu->PC,&cpu->Y,&cpu->X,cpu->memory_map);
    break;

  case 0x99: //STA $aaa&cpu->A,Y
    stx_indx(&cpu->PC,&cpu->A,&cpu->Y,cpu->memory_map);
    break;
  case 0x9D: //STA $aaa&cpu->A,X
    stx_indx(&cpu->PC,&cpu->A,&cpu->X,cpu->memory_map);
    break;

  case 0xAA: //TAX
    t_a2b(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xA8: //TAY
    t_a2b(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0x8A: //TXA
    t_a2b(&cpu->PC,&cpu->X,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x98: //TYA
    t_a2b(&cpu->PC,&cpu->Y,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0xBA: //TSX
    t_a2b(&cpu->PC,&cpu->SP,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x9A: //TXS  Esta instrucion parece que no afecta al SR
    SR_aux = cpu->P;
    t_a2b(&cpu->PC,&cpu->X,&cpu->SP,&cpu->P,cpu->memory_map);
    cpu->P = SR_aux;
    break;


  case 0xA9: //LDA #aa
    ldi(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0xA5: //LDA $aa
    ldx_zero(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0xB5: //LDA $a&cpu->A,X
    ld_indxZ(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xAD: //LDA $aaaa
    ldx(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0xB9: //LDA $aaa&cpu->A,Y
    ldx_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0xBD: //LDA $aaa&cpu->A,X
    ldx_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xA1: //LDA ($a&cpu->A,X)
    ldx_pre_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xB1: //LDA ($aa), Y
    ld_post_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;

  case 0xA2: //LDX #aa
    ldi(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xA6: //LDX $aa
    ldx_zero(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xB6: //LDX $a&cpu->A,Y
    ld_indxZ(&cpu->PC,&cpu->X,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0xAE: //LDX $aaaa
    ldx(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xBE: //LDX $aaa&cpu->A, Y
    ldx_indx(&cpu->PC,&cpu->X,&cpu->Y,&cpu->P,cpu->memory_map);
    break;

  case 0xA0: //LDY #aa
    ldi(&cpu->PC,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0xA4: //LDY $aa
    ldx_zero(&cpu->PC,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0xB4: //LDY $a&cpu->A,X
    ld_indxZ(&cpu->PC,&cpu->Y,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xAC: //LDY $aaaa
    ldx(&cpu->PC,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0xBC: //LDY $aaa&cpu->A, X
    ldx_indx(&cpu->PC,&cpu->Y,&cpu->X,&cpu->P,cpu->memory_map);
    break;


  case 0xEA: //NOP
    cpu->PC.uns++;
    break;
  case 0x1A: //NOP
    cpu->PC.uns++;
    break;
  case 0x3A: //NOP
    cpu->PC.uns++;
    break;
  case 0x5A: //NOP
    cpu->PC.uns++;
    break;
  case 0x7A: //NOP
    cpu->PC.uns++;
    break;
  case 0xDA: //NOP
    cpu->PC.uns++;
    break;
  case 0xFA : //NOP
    cpu->PC.uns++;
    break;

  case 0x0C: //NOP
    cpu->PC.uns = cpu->PC.uns + 3;
    break;
  case 0x1C: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0x3C: //NOP
    cpu->PC.uns = cpu->PC.uns + 3;
    break;
  case 0x5C: //NOP
    cpu->PC.uns = cpu->PC.uns + 3;
    break;
  case 0x7C: //NOP
    cpu->PC.uns = cpu->PC.uns + 3;
    break;
  case 0xDC: //NOP
    cpu->PC.uns = cpu->PC.uns + 3;
    break;
  case 0xFC: //NOP
    cpu->PC.uns = cpu->PC.uns + 3;
    break;

  case 0x80: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0x82: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0x89: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0xC2:
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0xE2: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0x04: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0x14: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0x34: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0x54: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0x74: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0xD4: //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;
  case 0xF4 : //NOP
    cpu->PC.uns = cpu->PC.uns + 2;
    break;


  case 0xE8: //INX
    inc(&cpu->PC,&cpu->X,&cpu->P);
    break;
  case 0xC8: //INY
    inc(&cpu->PC,&cpu->Y,&cpu->P);
    break;

    //SBC
  case 0xE9: //SBC #aa
    sub_i(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0xE5: //SBC aa
    sub_z(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0xED: //SBC aaaa
    sub_dir(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0xF5: //SBC a&cpu->A,X
    sub_indxz(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xFD: //SBC aaa&cpu->A,X
    sub_ind(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xF9: //SBC aaa&cpu->A,Y
    sub_ind(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0xF1: //SBC ($aa), Y
    //printf("hola voy a ejecutar\n");
    sub_post_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;

    //ADC
  case 0x69: //ADC #aa
    add_i(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x65: //ADC aa
    add_z(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x6D: //ADC aaaa
    //printf("hola");
    add_dir(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x75: //ADC a&cpu->A,X
    add_indxz(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x7D: //ADC aaa&cpu->A,X
    add_ind(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x79: //ADC aaa&cpu->A,Y
    add_ind(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0x71: //ADC ($aa), Y
    adc_post_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
    //INC

  case 0xE6: //INC aa
    inc_z(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0xF6: //INC a&cpu->A,X
    inc_indxz(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xEE: //INC aaaa
    inc_dir(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0xFE: //INC aaa&cpu->A,X
    inc_ind(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;

    //DEC

  case 0xC6: //DEC aa
    dec_z(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0xD6: //DEC a&cpu->A,X
    dec_indxz(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0xCE: //DEC aaaa
    dec_dir(&cpu->PC,&cpu->P,cpu->memory_map);
    break;
  case 0xDE: //DEC aaa&cpu->A,X
    dec_ind(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    break;


  case 0x88: //DEY
    dec(&cpu->PC,&cpu->Y,&cpu->P);
    break;
  case 0xCA: //DEX
    dec(&cpu->PC,&cpu->X,&cpu->P);
    break;

    //LOS OPCODES MISTERIOSOS
    //ASOs (SLO)
  case 0x07: //ASO aa
    PC_Aux = cpu->PC;
    asl_z(&cpu->PC,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    or_zero(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x17: //ASO a&cpu->A, X
    PC_Aux = cpu->PC;
    asl_xz(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    or_indxz(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x0F: //ASO aaaa
    PC_Aux = cpu->PC;
    asl(&cpu->PC,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    f_or(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x1F: //ASO aaa&cpu->A, X
    PC_Aux = cpu->PC;
    asl_indx(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    or_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x1B: //ASO aaa&cpu->A, Y
    PC_Aux = cpu->PC;
    asl_indx(&cpu->PC,&cpu->Y,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    or_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;
  case 0x03: //ASO (a&cpu->A, X) //Se icpu->memory_maplemento muy rapido no se sit ara bien
    PC_Aux = cpu->PC;
    asl_pre_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    or_pre_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;

    //RLAs
  case 0x27: //RLA aa
    PC_Aux = cpu->PC;
    rol_z(&cpu->PC,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    and_zero(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x37: //RLA a&cpu->A, X
    PC_Aux = cpu->PC;
    rol_xz(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    and_indxz(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x2F: //RLA aaaa
    PC_Aux = cpu->PC;
    rol(&cpu->PC,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    fand(&cpu->PC,&cpu->A,&cpu->P,cpu->memory_map);
    break;
  case 0x3F: //RLA aaa&cpu->A, X
    PC_Aux = cpu->PC;
    rol_indx(&cpu->PC,&cpu->X,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    and_indx(&cpu->PC,&cpu->A,&cpu->X,&cpu->P,cpu->memory_map);
    break;
  case 0x3B: //RLA aaa&cpu->A, Y
    PC_Aux = cpu->PC;
    rol_indx(&cpu->PC,&cpu->Y,&cpu->P,cpu->memory_map);
    cpu->PC = PC_Aux;
    and_indx(&cpu->PC,&cpu->A,&cpu->Y,&cpu->P,cpu->memory_map);
    break;

    //DCP
  case 0xCF: //DCP $aaaa
    PC_Aux = cpu->PC;
    dec_dir(&cpu->PC, &cpu->P, cpu->memory_map);
    cpu->PC = PC_Aux;
    cmp(&cpu->PC, &cpu->A, &cpu->P, cpu->memory_map);
    break;
  case 0xDF: //DCP $aaa&cpu->A, X
    PC_Aux = cpu->PC;
    dec_ind(&cpu->PC, &cpu->X, &cpu->P, cpu->memory_map);
    cpu->PC = PC_Aux;
    cmp_ind(&cpu->PC, &cpu->A, &cpu->X, &cpu->P, cpu->memory_map);
    break;
  case 0xDB: //DCP $aaa&cpu->A, Y
    PC_Aux = cpu->PC;
    dec_ind(&cpu->PC, &cpu->Y, &cpu->P, cpu->memory_map);
    cpu->PC = PC_Aux;
    cmp_ind(&cpu->PC, &cpu->A, &cpu->Y, &cpu->P, cpu->memory_map);
    break;

  default:
    printf("This OPCODE (%p) is not icpu->memory_maplemented yet or is invalid.\n", cpu->IR.uns);
    printf("PC was pointing: %p\n\n", cpu->PC.uns);
    printf("                     ***EXECUTION ABORTED***\n\n");
    halt[0] = 1;
    //exit(0);
    break;
  }
  cpu->memory_map = generar_excepciones(cpu);
  //} //FIN FOR
  return 4;

}


inline union tipo_palabra8* generar_excepciones(struct cpu6502* cpu)

{
  if (cpu->reset == 1)
  {
    //       cpu->memory_map = montar(cartucho, cpu->memory_map); //Se vuelve a motnar el cartucho.
    cpu->PC.uns = 256*cpu->memory_map[0xFFFD].uns + cpu->memory_map[0xFFFC].uns;
    poner_bit(2,1,&cpu->P); //Estamos en una INT
    cpu->reset = 0;
  }
  else if (cpu->nmi == 1)
  {
    cpu->memory_map[0x100 + cpu->SP.uns].uns = ((cpu->PC.uns)>>8);
    cpu->SP.uns--;
    cpu->memory_map[0x100+cpu->SP.uns].uns = cpu->PC.uns % 256;
    cpu->SP.uns--;
    cpu->memory_map[0x100+cpu->SP.uns].uns = cpu->P.uns & 0xEF;//Guardamos con B a 0.
    cpu->SP.uns--;



    cpu->PC.uns = 256*cpu->memory_map[0xFFFB].uns + cpu->memory_map[0xFFFA].uns;
    poner_bit(2,1,&cpu->P); //Estamos en una INT
    cpu->nmi = 0;

  }
  else if (((cpu->brk_irq == 1) || (cpu->brk_irq == 2)) &&    //Si hay brk o irq
           ((cpu->P.uns & 0x04 == 0) || cpu->brk_irq == 2))  //y no tan inhibidas INT o es brk
  {
    cpu->memory_map[0x100 + cpu->SP.uns].uns = cpu->PC.uns/256;
    cpu->SP.uns--;
    cpu->memory_map[0x100+cpu->SP.uns].uns = cpu->PC.uns % 256;
    cpu->SP.uns--;
    if (cpu->brk_irq == 2) {poner_bit(4,1,&cpu->P);}  //Si es BRK se pone el bit B a 1.
    else poner_bit(4,0,&cpu->P); //sino a 0.
    cpu->memory_map[0x100+cpu->SP.uns].uns = cpu->P.uns;
    cpu->SP.uns--;

    cpu->PC.uns = 256*cpu->memory_map[0xFFFF].uns + cpu->memory_map[0xFFFE].uns;
    poner_bit(2,1,&cpu->P); //Estamos en una INT
    cpu->brk_irq = 0;
  }



  return cpu->memory_map;

}


void reset6502(struct cpu6502* cpu)

{
  cpu->SP.uns = 0xFF;
  cpu->P.uns = 0x22;
  cpu->X.uns = 0;
  cpu->Y.uns = 0;
  cpu->A.uns = 0;

  cpu->PC.uns = 256*cpu->memory_map[0xFFFD].uns + cpu->memory_map[0xFFFC].uns; //Encendemos el PC
  cpu->reset = 0;
  cpu->brk_irq = 0;
  cpu->nmi = 0;
}


