#include "sdl_emilnes.h"
#include "palabras.c"
#include "ppu_state.h"
#include <stdlib.h>
#include "constantes.h"
#include "ppu.h"

unsigned int tabla_RGB[64][2];

unsigned char nes_palette[64][3] =
{
   {0x80,0x80,0x80}, {0x00,0x00,0xBB}, {0x37,0x00,0xBF}, {0x84,0x00,0xA6},
   {0xBB,0x00,0x6A}, {0xB7,0x00,0x1E}, {0xB3,0x00,0x00}, {0x91,0x26,0x00},
   {0x7B,0x2B,0x00}, {0x00,0x3E,0x00}, {0x00,0x48,0x0D}, {0x00,0x3C,0x22},
   {0x00,0x2F,0x66}, {0x00,0x00,0x00}, {0x05,0x05,0x05}, {0x05,0x05,0x05},

   {0xC8,0xC8,0xC8}, {0x00,0x59,0xFF}, {0x44,0x3C,0xFF}, {0xB7,0x33,0xCC},
   {0xFF,0x33,0xAA}, {0xFF,0x37,0x5E}, {0xFF,0x37,0x1A}, {0xD5,0x4B,0x00},
   {0xC4,0x62,0x00}, {0x3C,0x7B,0x00}, {0x1E,0x84,0x15}, {0x00,0x95,0x66},
   {0x00,0x84,0xC4}, {0x11,0x11,0x11}, {0x09,0x09,0x09}, {0x09,0x09,0x09},

   {0xFF,0xFF,0xFF}, {0x00,0x95,0xFF}, {0x6F,0x84,0xFF}, {0xD5,0x6F,0xFF},
   {0xFF,0x77,0xCC}, {0xFF,0x6F,0x99}, {0xFF,0x7B,0x59}, {0xFF,0x91,0x5F},
   {0xFF,0xA2,0x33}, {0xA6,0xBF,0x00}, {0x51,0xD9,0x6A}, {0x4D,0xD5,0xAE},
   {0x00,0xD9,0xFF}, {0x66,0x66,0x66}, {0x0D,0x0D,0x0D}, {0x0D,0x0D,0x0D},

   {0xFF,0xFF,0xFF}, {0x84,0xBF,0xFF}, {0xBB,0xBB,0xFF}, {0xD0,0xBB,0xFF},
   {0xFF,0xBF,0xEA}, {0xFF,0xBF,0xCC}, {0xFF,0xC4,0xB7}, {0xFF,0xCC,0xAE},
   {0xFF,0xD9,0xA2}, {0xCC,0xE1,0x99}, {0xAE,0xEE,0xB7}, {0xAA,0xF7,0xEE},
   {0xB3,0xEE,0xFF}, {0xDD,0xDD,0xDD}, {0x11,0x11,0x11}, {0x11,0x11,0x11}
};

void scan_line(void *screen, union tipo_palabra8* mp, int mir,
               int line);


void reset_ppu(struct ppu_data* ppu, void* pantalla, union tipo_palabra8** p_mp)
{
  ppu->frames = 0;
  ppu->mod_mal = 1;
  ppu->ciclo_us = SYNC_CPU_PPU; //Ciclos desde ultima sincronizacion(corre primero la PPU)
  ppu->line = 0;
  ppu->p_mp = p_mp;
  ppu->pantalla = pantalla;
}

void inline ejecutar_ppu(struct ppu_data* ppu, int* nmi)
{
  if ((ppu->ciclo_us >= ppu->p_mp[0][SPR_HIT].uns) &&
      (ppu->p_mp[0][SPR_HIT].uns > 16/3))  //Si es el momento del sprite hit...
    sprite_found(1,ppu->p_mp[0]);         //Pos se hace Sprite Hit

  if (ppu->ciclo_us == SYNC_CPU_PPU)

  {//PPU

    ppu->ciclo_us = 0; //Acabamos de sincornizar
    if (ppu->line == 241)//Esto se ha cambiado para simular la NTCS
    {ppu->line = 0;
      if ((ppu->p_mp[0][0x8FF].uns != 0) &&  ((ppu->frames % ppu->mod_mal) == 0))
      {
        update_screen(ppu->pantalla);
        if (ppu->p_mp[0][0x8FF].uns)ppu->p_mp[0][0x8FF].uns = 0;//Actualizado
      }
      ppu->frames++;
      poner_VBlank(1,ppu->p_mp[0]);
      nmi[0] = nmi_en_vblank(ppu->p_mp[0]);
    }
    else
    {
      if ((ppu->p_mp[0][0x8FF].uns != 0) &&  ((ppu->frames % ppu->mod_mal) == 0))
      {
        scan_line(ppu->pantalla,ppu->p_mp[0],ppu->p_mp[0][MIR].uns,ppu->line);
      }
      ppu->line++;
    }

  }
}


unsigned int color64(int n, void *screen) //dewelve un UnsignedInt32

{

  n = n % 64;
  //return SDL_MapRGB(screen->format,nes_palette[n][0] ,nes_palette[n][1], nes_palette[n][2]);
  if (tabla_RGB[n][1]==1)
    return tabla_RGB[n][0];
  else /*No esta calculada aun*/
  {
    tabla_RGB[n][1]=1;
    tabla_RGB[n][0]=RGB_a_UInt(screen,nes_palette[n][0] ,nes_palette[n][1], nes_palette[n][2]);
    return tabla_RGB[n][0];
  }
}

//FUNCIONES AUXILIARES PARA FACILITAR LA LECTURA

void dir_patron(int x, int line, union tipo_palabra16 name_table,
                union tipo_palabra16 patern_table,
                union tipo_palabra16* dir_pat,
                union tipo_palabra8* mp, int mir)

{
  union tipo_palabra16 dir;
  union tipo_palabra8 aux1;

  dir.uns = name_table.uns + (x >> 3) + 32*(line >> 3); //mirar en name table q patron
  leer_vram(&aux1,mp,dir,mir); //Num Patron
  dir_pat->uns = (aux1.uns << 4) + patern_table.uns; //2 bytes por patron

}

void fcolor_atrib(int x, int line,
                  unsigned int* color_atrib,
                  union tipo_palabra8* mp, int mir, union tipo_palabra16 base)
{
  union tipo_palabra16 dir;
  union tipo_palabra8 aux1;
  union tipo_palabra8 aux2;

  aux1.uns = (x >> 5) + 8*(line/32); //numero byte del atribute.
  leer_vram(&aux2,mp,base.uns+0x3C0+aux1.uns,mir); //Est�en este byte.
  if (x%32 < 16 ) //SQ0
  {if (((line) % 32) < 16) //SQ 0
    {
      //color_atrib[0] = 8*sacar_bit(1,aux2) + 4*sacar_bit(0,aux2); //0000 0011 -> 0000 1100
      *color_atrib = ((aux2.uns & 0x03) << 2);
    }
    else //SQ 2
    {
      //color_atrib[0] = 8*sacar_bit(5,aux2) + 4*sacar_bit(4,aux2); //0011 0000 -> 0000 1100
      *color_atrib = ((aux2.uns & 0x30) >> 2);
    }
  }
  else //otros SQ 1
  {
    if (((line) % 32) < 16) //SQ 1
    {
      //color_atrib[0] = 8*sacar_bit(3,aux2) + 4*sacar_bit(2,aux2); //0000 1100 -> 0000 1100
      *color_atrib = (aux2.uns & 0x0C);
    }
    else //SQ 3
    {
      //color_atrib[0] = 8*sacar_bit(7,aux2) + 4*sacar_bit(6,aux2); //1100 0000 -> 0000 1100
      *color_atrib = ((aux2.uns & 0xC0) >> 4);
    }

  }


}


void ponerpixel(void *screen, int x, int line, unsigned int color)

{
  /*       if ( SDL_MUSTLOCK(screen) ) {
  if ( SDL_LockSurface(screen) < 0 ) 
      {
      fprintf(stderr, "Can't lock screen: %s\n", SDL_GetError());
      }
   }
   */
  //if ((x < 0) || (x > 256) || (line < 0) || (line > 240))
  //   printf("Se kiere pintar fuera de rango");

  putpixel(screen,x+32,line,color);
  /*if ( SDL_MUSTLOCK(screen) ) SDL_UnlockSurface(screen);*/

}

void para_dibujar(union tipo_palabra8* mp, unsigned char line) //indica que sprites se dibujaran en este scanline
{
  int i;
  int cont = 0;

  //por cuestiones de tama�
  unsigned int lineB = line;
  unsigned int Y;
  unsigned int tam = 8;

  if ((mp[0x2000].uns & 0x20) == 0x20) {tam = 16;}

  if (display_sprites(mp) == 0) //No hay sprites que dibujar
  {mp[0x870].sig = -1;
    return;
  }
  //else
  //for (i = 0; i<64; i++)
  for (i = 63; i >= 0; i--)
  {
    Y = mp[0x900+4*i].uns + 1;//XQ es Y - 1;
    //if (sacar_bit(5,mp[0x900+4*i+2]) == 1) ;//no se dibujara, est�detras.
    /*else*/ if ((lineB >= Y) && (lineB < Y + tam))
    {
      mp[0x870 + cont].uns = i;
      cont++;
    }
  }
  mp[0x870 + cont].sig = -1; //Marcamos el final
  //if (mp[0x870].sig == -1) {printf("en %i no sprite\n", line); system("pause");}
}

flipear(union tipo_palabra8* offset_x,union tipo_palabra8* offset_y,
        int q_dibujar,union tipo_palabra8* mp)

{
  if ((mp[0x900 + 4*q_dibujar + 2].uns & 0x80) == 0x80)
    offset_y[0].uns = 7 - offset_y[0].uns; //ATENCION: No independiente
  if ((mp[0x900 + 4*q_dibujar + 2].uns & 0x40) == 0x40)
    offset_x[0].uns = 7 - offset_x[0].uns; //ATENCION: No independiente

}


unsigned int nueva_nt_v(union tipo_palabra16 name_table)
{
  if (name_table.uns == 0x2000)
    return 0x2800;
  else if (name_table.uns == 0x2800)
    return 0x2000;
  else if (name_table.uns == 0x2C00)
    return 0x2400;
  else return 0x2C00;
}

unsigned int nueva_nt_h(union tipo_palabra16 name_table)
{
  if (name_table.uns == 0x2000)
    return 0x2400;
  else if (name_table.uns == 0x2800)
    return 0x2C00;
  else if (name_table.uns == 0x2C00)
    return 0x2800;
  else return 0x2000;
}

void scan_bckg(void *screen, union tipo_palabra8* mp, int mir,
               unsigned char line_fisica)

{
  mp[SPR_HIT].uns = 0x0;

  int x;
  int x_fisica;
  unsigned char line;
  union tipo_palabra16 patern_table;
  union tipo_palabra16 patern_table_spr; //La de los sprites
  union tipo_palabra16 name_table;

  int posible_hit = 0; //de momento no hay hit
  int spr_behind = 0;

  name_table.uns = name_t(mp);
  patern_table.uns = screen_pt(mp); //Tenemos la dir del patern_table
  patern_table_spr.uns = sprite_pt(mp); //La dir para los sprites.
  int q_dibujar; //Si -1 back_ground sino es el #sprite que dibujar
  int contador_array;

  //vars auxiliares
  union tipo_palabra8 aux1;
  union tipo_palabra8 aux2;

  union tipo_palabra8 aux3;
  union tipo_palabra8 aux4;

  union tipo_palabra8 offset_y; //Respecto al patron a dibujar de sprite
  union tipo_palabra8 offset_x;
  union tipo_palabra16 dir;

  unsigned int color_atrib; //Pa leer una sola vez la tbla de atributos
  union tipo_palabra16 dir_pat; //Direccion del patron para leerle solo una vez tb
  unsigned int color;

  unsigned int pre_colores [8];
  //union tipo_palabra8 color_fondo = 0;
  int i;
  //FUNCIONES-MACRO PARA AGILIZAR LA LECTURA
  void inline f_adelantar()
  {

    leer_vram(&aux3,mp,dir_pat.uns+8+((line%8)*1),mir);
    leer_vram(&aux4,mp,dir_pat.uns+((line%8)*1),mir);

  }
  //FIN FUNCIONES MACRO

  //printf("El scroll_H es %i y el V es %i\n", mp[SCROLL_H].uns, mp[SCROLL_V].uns);

  //AJUSTE DEL SCROLL HORIZONTAL
  line = (line_fisica + mp[SCROLL_V].uns) % 240;
  if (line_fisica + mp[SCROLL_V].uns > 239)
    name_table.uns = nueva_nt_v(name_table);

  for(x_fisica = 0; x_fisica < 255; x_fisica++)

  {
    x = (x_fisica + mp[SCROLL_H].uns) % 256;
    if (x_fisica + mp[SCROLL_H].uns > 255)
      name_table.uns = nueva_nt_h(name_table);

    //if (mp[0x900].sig == -1)printf("NO Hay sprite para dibujar");
    // if (line == 32) printf("%i\n",x);
    //PATRON DE ESCENARIO

    if ((x % 8) == 0) //se carga el patron de tabla
    {dir_patron(x,line,name_table,patern_table,&dir_pat,mp,mir);
      //FIN PATRON

      //TABLA DE ATRIBUTOS DEL BIT DE FONDO

      if ((x % 16) == 0) //se cargan los 2 bit del atribute
      {fcolor_atrib(x,line,&color_atrib,mp,mir,name_table);

      }//fin if
    }//fin primer if

    //FIN ATRIBUTO

    //Y AHORA ES EL MOMENTO DE VER SI TENEMOS QUE PONER UN BACKGROUND O
    //UN HERMOSOTE SPRITE

    contador_array = 0;
    while (1)
    {
      aux1 = mp[0x870 + contador_array];
      if (aux1.sig == -1)
      {
        q_dibujar = -1;
        break;
      }
      else if ( (mp[0x900 + 4*aux1.sig + 3].uns <= x_fisica) &&
                (mp[0x900 + 4*aux1.sig + 3].uns + 7 >= x_fisica ) )//Esta ecuacion se tiene q comprobar
      {q_dibujar = aux1.sig;
        break;
      }
      contador_array++;
    }


    //YA SABEMOS LO QUE TENEMOS

    //DIBUJO DEL PIXEL EN PANTALLA


    if (x%8 == 0) f_adelantar(); //ADELANTAMOS EL RENDER PARA SER MAS EFICIENTES

    //APROXIMACION SPR_HIT 0
    if (q_dibujar == 0)
    {mp[SPR_HIT].uns = (x_fisica + 16) / 3;}

    //DIBUJAR SPRITES
    if (q_dibujar != -1) //&& (display_sprites(mp) == 1)) // se tendra que dibujar un sprite
    {

      offset_y.uns = line_fisica - mp[0x900 + 4*q_dibujar].uns - 1;
      offset_x.uns = x_fisica - mp[0x900 + 4*q_dibujar + 3].uns;

      flipear(&offset_x, &offset_y, q_dibujar, mp);

      dir.uns = patern_table_spr.uns + 16*mp[0x900 + q_dibujar*4 + 1].uns; //La dir el patron

      leer_vram(&aux1,mp,dir.uns + 8 + offset_y.uns,mir);
      leer_vram(&aux2,mp,dir.uns + offset_y.uns,mir);

      dir.uns = (mp[0x900 + 4*q_dibujar + 2].uns % 4)* 4 //Dos bits de SPR-RAM
                + 2*sacar_bit(7 - offset_x.uns,aux1)            //Dos bit del patron
                + sacar_bit(7 - offset_x.uns,aux2)
                + 0x3F10;//printf("sprite\n");           //Paleta de Sprites

      if (sacar_bit(5,mp[0x900+4*q_dibujar+2]) == 1)
      {
        spr_behind = 1; //Como es behind...
        q_dibujar = -1; //Para q entre el fondo
      }

      //En caso de q no sea fondo podra haber spr_hit
      //if ((q_dibujar == 0) && (dir.uns % 4 != 0)) {//sprite_found(1,mp);
      //                     posible_hit = 1;
      //                    }

      leer_vram(&aux1,mp,dir,mir); //Miramos en la paleta

      if (dir.uns % 4 == 0x0) //Si transparente
      {
        q_dibujar = -1;
      }

      else
      {
        color = color64(aux1.uns,screen); //Conseguimos el color RGB
        ponerpixel(screen,x_fisica,line_fisica,color);
      }
    }

    if (q_dibujar == -1)
      // ESTO ES PARA DIBUJAR EL FONDO

      {    if (display_screen(mp) == 1)
      {
        dir.uns = 2*sacar_bit(7 - (x%8),aux3)
                  + sacar_bit(7 - (x%8),aux4)
                  + color_atrib + 0x3F00;
      }
      else dir.uns = 0x3F00;

      //DETERMINAMOS SI HAY HIT O NO
      //if ((dir.uns % 4!= 0) && (posible_hit == 1))
      //   mp[SPR_HIT].uns = (x_fisica + 16) / 3;

      if ((dir.uns % 4) == 0) leer_vram(&aux1,mp,0x3f00,mir); //Son trasparentes los mod 4.
      else leer_vram(&aux1,mp,dir,mir); //Miramos en la paleta

      color = color64(aux1.uns,screen); //Conseguimos el color RGB

      if ((dir.uns == 0x3f00) && (spr_behind == 1)) //si no transparente y spr behind
        ; //NIL

      else
        //PONER PIXEL SDL
        ponerpixel(screen,x_fisica,line_fisica,color);
      //FIN PONER PIXEL SDL
      spr_behind = 0; //Aseguremonos
    }

  }


}



void scan_line(void *screen, union tipo_palabra8* mp, int mir,
               int line)

{
  para_dibujar(mp, line);
  if ((line > 8) && (line < 232)) scan_bckg(screen,mp,mir,line);

  return;
}


void inline dma_sim(union tipo_palabra8* mp)
{
  static union tipo_palabra16 dir_dma;
  static union tipo_palabra8 aux_dma;

  if (mp[0xA03].uns == 1)
  {//Calculamos la direccion de la que hay q leer
    //100*N + Offset
    dir_dma.uns = 256*mp[0xA02].uns + mp[0xA01].uns;
    //Scamos el dato a copiar
    leer_mp(&aux_dma,mp,dir_dma);
    mp[0x900 + mp[0xA01].uns]= aux_dma;

    mp[0xA01].uns++;

    if (mp[0xA01].uns == 0) mp[0xA03].uns = 0; //Si da el voltio acabar.

  }
}


